@@ -1,7 +1,7 @@
 /* mpz_xor -- Logical xor.
 
-Copyright 1991, 1993, 1994, 1996, 1997, 2000, 2001, 2005 Free Software
-Foundation, Inc.
+Copyright 1991, 1993, 1994, 1996, 1997, 2000, 2001, 2005, 2012 Free
+Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -28,16 +28,15 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
   mp_size_t op1_size, op2_size;
   mp_ptr res_ptr;
   mp_size_t res_size, res_alloc;
-  mp_size_t i;
   TMP_DECL;
 
   TMP_MARK;
-  op1_size = op1->_mp_size;
-  op2_size = op2->_mp_size;
+  op1_size = SIZ(op1);
+  op2_size = SIZ(op2);
 
-  op1_ptr = op1->_mp_d;
-  op2_ptr = op2->_mp_d;
-  res_ptr = res->_mp_d;
+  op1_ptr = PTR(op1);
+  op2_ptr = PTR(op2);
+  res_ptr = PTR(res);
 
   if (op1_size >= 0)
     {
@@ -45,41 +44,41 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
 	{
 	  if (op1_size >= op2_size)
 	    {
-	      if (res->_mp_alloc < op1_size)
+	      if (ALLOC(res) < op1_size)
 		{
 		  _mpz_realloc (res, op1_size);
-		  op1_ptr = op1->_mp_d;
-		  op2_ptr = op2->_mp_d;
-		  res_ptr = res->_mp_d;
+		  /* No overlapping possible: op1_ptr = PTR(op1); */
+		  op2_ptr = PTR(op2);
+		  res_ptr = PTR(res);
 		}
 
 	      if (res_ptr != op1_ptr)
 		MPN_COPY (res_ptr + op2_size, op1_ptr + op2_size,
 			  op1_size - op2_size);
-	      for (i = op2_size - 1; i >= 0; i--)
-		res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
+	      if (LIKELY (op2_size != 0))
+		mpn_xor_n (res_ptr, op1_ptr, op2_ptr, op2_size);
 	      res_size = op1_size;
 	    }
 	  else
 	    {
-	      if (res->_mp_alloc < op2_size)
+	      if (ALLOC(res) < op2_size)
 		{
 		  _mpz_realloc (res, op2_size);
-		  op1_ptr = op1->_mp_d;
-		  op2_ptr = op2->_mp_d;
-		  res_ptr = res->_mp_d;
+		  op1_ptr = PTR(op1);
+		  /* No overlapping possible: op2_ptr = PTR(op2); */
+		  res_ptr = PTR(res);
 		}
 
 	      if (res_ptr != op2_ptr)
 		MPN_COPY (res_ptr + op1_size, op2_ptr + op1_size,
 			  op2_size - op1_size);
-	      for (i = op1_size - 1; i >= 0; i--)
-		res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
+	      if (LIKELY (op1_size != 0))
+		mpn_xor_n (res_ptr, op1_ptr, op2_ptr, op1_size);
 	      res_size = op2_size;
 	    }
 
 	  MPN_NORMALIZE (res_ptr, res_size);
-	  res->_mp_size = res_size;
+	  SIZ(res) = res_size;
 	  return;
 	}
       else /* op2_size < 0 */
@@ -91,7 +90,7 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
     {
       if (op2_size < 0)
 	{
-	  mp_ptr opx;
+	  mp_ptr opx, opy;
 
 	  /* Both operands are negative, the result will be positive.
 	      (-OP1) ^ (-OP2) =
@@ -103,43 +102,26 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
 
 	  /* Possible optimization: Decrease mpn_sub precision,
 	     as we won't use the entire res of both.  */
-	  opx = (mp_ptr) TMP_ALLOC (op1_size * BYTES_PER_MP_LIMB);
+	  TMP_ALLOC_LIMBS_2 (opx, op1_size, opy, op2_size);
 	  mpn_sub_1 (opx, op1_ptr, op1_size, (mp_limb_t) 1);
 	  op1_ptr = opx;
 
-	  opx = (mp_ptr) TMP_ALLOC (op2_size * BYTES_PER_MP_LIMB);
-	  mpn_sub_1 (opx, op2_ptr, op2_size, (mp_limb_t) 1);
-	  op2_ptr = opx;
-
-	  res_alloc = MAX (op1_size, op2_size);
-	  if (res->_mp_alloc < res_alloc)
-	    {
-	      _mpz_realloc (res, res_alloc);
-	      res_ptr = res->_mp_d;
-	      /* Don't re-read OP1_PTR and OP2_PTR.  They point to
-		 temporary space--never to the space RES->_mp_d used
-		 to point to before reallocation.  */
-	    }
+	  mpn_sub_1 (opy, op2_ptr, op2_size, (mp_limb_t) 1);
+	  op2_ptr = opy;
 
 	  if (op1_size > op2_size)
-	    {
-	      MPN_COPY (res_ptr + op2_size, op1_ptr + op2_size,
-			op1_size - op2_size);
-	      for (i = op2_size - 1; i >= 0; i--)
-		res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
-	      res_size = op1_size;
-	    }
-	  else
-	    {
-	      MPN_COPY (res_ptr + op1_size, op2_ptr + op1_size,
-			op2_size - op1_size);
-	      for (i = op1_size - 1; i >= 0; i--)
-		res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
-	      res_size = op2_size;
-	    }
+	    MPN_SRCPTR_SWAP (op1_ptr,op1_size, op2_ptr,op2_size);
+
+	  res_alloc = op2_size;
+	  res_ptr = MPZ_REALLOC (res, res_alloc);
+
+	  MPN_COPY (res_ptr + op1_size, op2_ptr + op1_size,
+		    op2_size - op1_size);
+	  mpn_xor_n (res_ptr, op1_ptr, op2_ptr, op1_size);
+	  res_size = op2_size;
 
 	  MPN_NORMALIZE (res_ptr, res_size);
-	  res->_mp_size = res_size;
+	  SIZ(res) = res_size;
 	  TMP_FREE;
 	  return;
 	}
@@ -147,8 +129,8 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
 	{
 	  /* We should compute -OP1 ^ OP2.  Swap OP1 and OP2 and fall
 	     through to the code that handles OP1 ^ -OP2.  */
-          MPZ_SRCPTR_SWAP (op1, op2);
-          MPN_SRCPTR_SWAP (op1_ptr,op1_size, op2_ptr,op2_size);
+	  MPZ_SRCPTR_SWAP (op1, op2);
+	  MPN_SRCPTR_SWAP (op1_ptr,op1_size, op2_ptr,op2_size);
 	}
     }
 
@@ -163,44 +145,39 @@ mpz_xor (mpz_ptr res, mpz_srcptr op1, mpz_srcptr op2)
 
     op2_size = -op2_size;
 
-    opx = (mp_ptr) TMP_ALLOC (op2_size * BYTES_PER_MP_LIMB);
+    opx = TMP_ALLOC_LIMBS (op2_size);
     mpn_sub_1 (opx, op2_ptr, op2_size, (mp_limb_t) 1);
     op2_ptr = opx;
 
     res_alloc = MAX (op1_size, op2_size) + 1;
-    if (res->_mp_alloc < res_alloc)
+    if (ALLOC(res) < res_alloc)
       {
 	_mpz_realloc (res, res_alloc);
-	op1_ptr = op1->_mp_d;
-	res_ptr = res->_mp_d;
-	/* Don't re-read OP2_PTR.  It points to temporary space--never
-	   to the space RES->_mp_d used to point to before reallocation.  */
+	op1_ptr = PTR(op1);
+	/* op2_ptr points to temporary space.  */
+	res_ptr = PTR(res);
       }
 
     if (op1_size > op2_size)
       {
 	MPN_COPY (res_ptr + op2_size, op1_ptr + op2_size, op1_size - op2_size);
-	for (i = op2_size - 1; i >= 0; i--)
-	  res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
+	mpn_xor_n (res_ptr, op1_ptr, op2_ptr, op2_size);
 	res_size = op1_size;
       }
     else
       {
 	MPN_COPY (res_ptr + op1_size, op2_ptr + op1_size, op2_size - op1_size);
-	for (i = op1_size - 1; i >= 0; i--)
-	  res_ptr[i] = op1_ptr[i] ^ op2_ptr[i];
+	if (LIKELY (op1_size != 0))
+	  mpn_xor_n (res_ptr, op1_ptr, op2_ptr, op1_size);
 	res_size = op2_size;
       }
 
     cy = mpn_add_1 (res_ptr, res_ptr, res_size, (mp_limb_t) 1);
-    if (cy)
-      {
-	res_ptr[res_size] = cy;
-	res_size++;
-      }
+    res_ptr[res_size] = cy;
+    res_size += (cy != 0);
 
     MPN_NORMALIZE (res_ptr, res_size);
-    res->_mp_size = -res_size;
+    SIZ(res) = -res_size;
     TMP_FREE;
   }
 }
