@@ -1,23 +1,22 @@
 /* Test mpz_setbit, mpz_clrbit, mpz_tstbit.
 
-Copyright 1997, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+Copyright 1997, 2000, 2001, 2002, 2003, 2012, 2013 Free Software
+Foundation, Inc.
 
-This file is part of the GNU MP Library.
+This file is part of the GNU MP Library test suite.
 
-The GNU MP Library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
-option) any later version.
+The GNU MP Library test suite is free software; you can redistribute it
+and/or modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 3 of the License,
+or (at your option) any later version.
 
-The GNU MP Library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-License for more details.
+The GNU MP Library test suite is distributed in the hope that it will be
+useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
+Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+You should have received a copy of the GNU General Public License along with
+the GNU MP Library test suite.  If not, see http://www.gnu.org/licenses/.  */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +39,7 @@ debug_mp (mpz_srcptr x, int base)
 
 /* exercise the case where mpz_clrbit or mpz_combit ends up extending a
    value like -2^(k*GMP_NUMB_BITS-1) when clearing bit k*GMP_NUMB_BITS-1.  */
+/* And vice-versa. */
 void
 check_clr_extend (void)
 {
@@ -53,38 +53,60 @@ check_clr_extend (void)
   for (i = 1; i < 5; i++)
     {
       for (f = 0; f <= 1; f++)
-        {
-          /* lots of 1 bits in _mp_d */
-          mpz_set_ui (got, 1L);
-          mpz_mul_2exp (got, got, 10*GMP_NUMB_BITS);
-          mpz_sub_ui (got, got, 1L);
+	{
+	  /* lots of 1 bits in _mp_d */
+	  mpz_set_si (got, 1L);
+	  mpz_mul_2exp (got, got, 10*GMP_NUMB_BITS);
+	  mpz_sub_ui (got, got, 1L);
 
-          /* value -2^(n-1) representing ..11100..00 */
-          mpz_set_si (got, -1L);
-          mpz_mul_2exp (got, got, i*GMP_NUMB_BITS-1);
+	  /* value -2^(n-1) representing ..11100..00 */
+	  mpz_set_si (got, -1L);
+	  mpz_mul_2exp (got, got, i*GMP_NUMB_BITS-1);
 
-          /* complement bit n, giving ..11000..00 which is -2^n */
-          if (f == 0)
-            mpz_clrbit (got, i*GMP_NUMB_BITS-1);
-          else
-            mpz_combit (got, i*GMP_NUMB_BITS-1);
-          MPZ_CHECK_FORMAT (got);
+	  /* complement bit n, giving ..11000..00 which is -2^n */
+	  if (f == 0)
+	    mpz_clrbit (got, i*GMP_NUMB_BITS-1);
+	  else
+	    mpz_combit (got, i*GMP_NUMB_BITS-1);
+	  MPZ_CHECK_FORMAT (got);
 
-          mpz_set_si (want, -1L);
-          mpz_mul_2exp (want, want, i*GMP_NUMB_BITS);
+	  mpz_set_si (want, -1L);
+	  mpz_mul_2exp (want, want, i*GMP_NUMB_BITS);
 
-          if (mpz_cmp (got, want) != 0)
-            {
-              if (f == 0)
-                printf ("mpz_clrbit: ");
-              else
-                printf ("mpz_combit: ");
-              printf ("wrong after extension\n");
-              mpz_trace ("got ", got);
-              mpz_trace ("want", want);
-              abort ();
-            }
-        }
+	  if (mpz_cmp (got, want) != 0)
+	    {
+	      if (f == 0)
+		printf ("mpz_clrbit: ");
+	      else
+		printf ("mpz_combit: ");
+	      printf ("wrong after extension\n");
+	      mpz_trace ("got ", got);
+	      mpz_trace ("want", want);
+	      abort ();
+	    }
+
+	  /* complement bit n, going back to ..11100..00 which is -2^(n-1) */
+	  if (f == 0)
+	    mpz_setbit (got, i*GMP_NUMB_BITS-1);
+	  else
+	    mpz_combit (got, i*GMP_NUMB_BITS-1);
+	  MPZ_CHECK_FORMAT (got);
+
+	  mpz_set_si (want, -1L);
+	  mpz_mul_2exp (want, want, i*GMP_NUMB_BITS - 1);
+
+	  if (mpz_cmp (got, want) != 0)
+	    {
+	      if (f == 0)
+		printf ("mpz_setbit: ");
+	      else
+		printf ("mpz_combit: ");
+	      printf ("wrong after shrinking\n");
+	      mpz_trace ("got ", got);
+	      mpz_trace ("want", want);
+	      abort ();
+	    }
+	}
     }
 
   mpz_clear (got);
@@ -126,14 +148,14 @@ check_com_negs (void)
       mpz_combit (got, data[i].bit);
 
       if (mpz_cmp (got, want) != 0)
-        {
-          printf ("mpz_combit: wrong on neg data[%d]\n", i);
-          mpz_trace ("inp ", inp);
-          printf    ("bit %lu\n", data[i].bit);
-          mpz_trace ("got ", got);
-          mpz_trace ("want", want);
-          abort ();
-        }
+	{
+	  printf ("mpz_combit: wrong on neg data[%d]\n", i);
+	  mpz_trace ("inp ", inp);
+	  printf    ("bit %lu\n", data[i].bit);
+	  mpz_trace ("got ", got);
+	  mpz_trace ("want", want);
+	  abort ();
+	}
     }
 
   mpz_clear (inp);
@@ -163,30 +185,30 @@ check_tstbit (void)
       mpn_random2 (pos+zeros, (mp_size_t) NUM_LIMBS);
 
       for (low1 = 0; low1 <= 1; low1++)
-        {
-          if (low1)
-            pos[0] |= 1;
+	{
+	  if (low1)
+	    pos[0] |= 1;
 
-          refmpn_neg_n (neg, pos, (mp_size_t) numberof(neg));
-          mpz_set_n (z, neg, (mp_size_t) numberof(neg));
-          mpz_neg (z, z);
+	  refmpn_neg (neg, pos, (mp_size_t) numberof(neg));
+	  mpz_set_n (z, neg, (mp_size_t) numberof(neg));
+	  mpz_neg (z, z);
 
-          for (i = 0; i < numberof(pos)*GMP_NUMB_BITS; i++)
-            {
-              got = mpz_tstbit (z, i);
-              want = refmpn_tstbit (pos, i);
-              if (got != want)
-                {
-                  printf ("wrong at bit %lu, with %d zeros\n", i, zeros);
-                  printf ("z neg "); debug_mp (z, -16);
-                  mpz_set_n (z, pos, (mp_size_t) numberof(pos));
-                  printf ("pos   "); debug_mp (z, -16);
-                  mpz_set_n (z, neg, (mp_size_t) numberof(neg));
-                  printf ("neg   "); debug_mp (z, -16);
-                  exit (1);
-                }
-            }
-        }
+	  for (i = 0; i < numberof(pos)*GMP_NUMB_BITS; i++)
+	    {
+	      got = mpz_tstbit (z, i);
+	      want = refmpn_tstbit (pos, i);
+	      if (got != want)
+		{
+		  printf ("wrong at bit %lu, with %d zeros\n", i, zeros);
+		  printf ("z neg "); debug_mp (z, -16);
+		  mpz_set_n (z, pos, (mp_size_t) numberof(pos));
+		  printf ("pos   "); debug_mp (z, -16);
+		  mpz_set_n (z, neg, (mp_size_t) numberof(neg));
+		  printf ("neg   "); debug_mp (z, -16);
+		  exit (1);
+		}
+	    }
+	}
     }
   mpz_clear (z);
 }
@@ -204,55 +226,55 @@ check_single (void)
   for (limb = 0; limb < 4; limb++)
     {
       for (offset = (limb==0 ? 0 : -2); offset <= 2; offset++)
-        {
-          for (initial = 0; initial >= -1; initial--)
-            {
-              mpz_set_si (x, (long) initial);
+	{
+	  for (initial = 1; initial >= -1; initial--)
+	    {
+	      mpz_set_si (x, (long) initial);
 
-              bit = (unsigned long) limb*BITS_PER_MP_LIMB + offset;
+	      bit = (unsigned long) limb*GMP_LIMB_BITS + offset;
 
-              mpz_clrbit (x, bit);
-              MPZ_CHECK_FORMAT (x);
-              if (mpz_tstbit (x, bit) != 0)
-                {
-                  printf ("check_single(): expected 0\n");
-                  abort ();
-                }
-          
-              mpz_setbit (x, bit);
-              MPZ_CHECK_FORMAT (x);
-              if (mpz_tstbit (x, bit) != 1)
-                {
-                  printf ("check_single(): expected 1\n");
-                  abort ();
-                }
-          
-              mpz_clrbit (x, bit);
-              MPZ_CHECK_FORMAT (x);
-              if (mpz_tstbit (x, bit) != 0)
-                {
-                  printf ("check_single(): expected 0\n");
-                  abort ();
-                }
+	      mpz_clrbit (x, bit);
+	      MPZ_CHECK_FORMAT (x);
+	      if (mpz_tstbit (x, bit) != 0)
+		{
+		  printf ("check_single(): expected 0\n");
+		  abort ();
+		}
 
-              mpz_combit (x, bit);
-              MPZ_CHECK_FORMAT (x);
-              if (mpz_tstbit (x, bit) != 1)
-                {
-                  printf ("check_single(): expected 1\n");
-                  abort ();
-                }
+	      mpz_setbit (x, bit);
+	      MPZ_CHECK_FORMAT (x);
+	      if (mpz_tstbit (x, bit) != 1)
+		{
+		  printf ("check_single(): expected 1\n");
+		  abort ();
+		}
 
-              mpz_combit (x, bit);
-              MPZ_CHECK_FORMAT (x);
-              if (mpz_tstbit (x, bit) != 0)
-                {
-                  printf ("check_single(): expected 0\n");
-                  abort ();
-                }
-            }
-        }
-    }          
+	      mpz_clrbit (x, bit);
+	      MPZ_CHECK_FORMAT (x);
+	      if (mpz_tstbit (x, bit) != 0)
+		{
+		  printf ("check_single(): expected 0\n");
+		  abort ();
+		}
+
+	      mpz_combit (x, bit);
+	      MPZ_CHECK_FORMAT (x);
+	      if (mpz_tstbit (x, bit) != 1)
+		{
+		  printf ("check_single(): expected 1\n");
+		  abort ();
+		}
+
+	      mpz_combit (x, bit);
+	      MPZ_CHECK_FORMAT (x);
+	      if (mpz_tstbit (x, bit) != 0)
+		{
+		  printf ("check_single(): expected 0\n");
+		  abort ();
+		}
+	    }
+	}
+    }
 
   mpz_clear (x);
 }
@@ -297,7 +319,7 @@ check_random (int argc, char *argv[])
 
       mpz_set (s2, x);
       bit2 = mpz_tstbit (x, bitindex);
-      mpz_setbit (x, bitindex);
+      mpz_combit (x, bitindex);
       MPZ_CHECK_FORMAT (x);
 
       mpz_set (s3, x);
@@ -325,16 +347,26 @@ check_random (int argc, char *argv[])
       if (mpz_cmp (s2, s3) == 0)
 	abort ();
 
+      mpz_combit (x, bitindex);
+      MPZ_CHECK_FORMAT (x);
+      if (mpz_cmp (s2, x) != 0)
+	abort ();
+
+      mpz_clrbit (x, bitindex);
+      MPZ_CHECK_FORMAT (x);
+      if (mpz_cmp (s2, x) != 0)
+	abort ();
+
       mpz_ui_pow_ui (m, 2L, bitindex);
       MPZ_CHECK_FORMAT (m);
-      mpz_ior (x, s2, m);
+      mpz_ior (x, s0, m);
       MPZ_CHECK_FORMAT (x);
       if (mpz_cmp (x, s3) != 0)
 	abort ();
 
       mpz_com (m, m);
       MPZ_CHECK_FORMAT (m);
-      mpz_and (x, s1, m);
+      mpz_and (x, s0, m);
       MPZ_CHECK_FORMAT (x);
       if (mpz_cmp (x, s2) != 0)
 	abort ();
@@ -373,5 +405,3 @@ main (int argc, char *argv[])
   tests_end ();
   exit (0);
 }
-
-
