@@ -6,26 +6,28 @@
 	mpz_mul_si
 	mpz_addmul_ui (should this really allow a+=a*c?)
 
-Copyright 1996, 1999, 2000, 2001, 2002, 2009 Free Software Foundation, Inc.
+Copyright 1996, 1999, 2000, 2001, 2002, 2009, 2012, 2013 Free Software
+Foundation, Inc.
 
-This file is part of the GNU MP Library.
+This file is part of the GNU MP Library test suite.
 
-The GNU MP Library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+The GNU MP Library test suite is free software; you can redistribute it
+and/or modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 3 of the License,
+or (at your option) any later version.
 
-The GNU MP Library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-License for more details.
+The GNU MP Library test suite is distributed in the hope that it will be
+useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
+Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
+You should have received a copy of the GNU General Public License along with
+the GNU MP Library test suite.  If not, see http://www.gnu.org/licenses/.  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "gmp.h"
 #include "gmp-impl.h"
@@ -48,14 +50,14 @@ main (void)
 
 #else /* ! DLL_EXPORT */
 
-void dump __GMP_PROTO ((char *, mpz_t, mpz_t, mpz_t));
+void dump (const char *, mpz_t, mpz_t, mpz_t);
 
-typedef void (*dss_func) __GMP_PROTO ((mpz_ptr, mpz_srcptr, mpz_srcptr));
-typedef void (*dsi_func) __GMP_PROTO ((mpz_ptr, mpz_srcptr, unsigned long int));
-typedef unsigned long int (*dsi_div_func) __GMP_PROTO ((mpz_ptr, mpz_srcptr, unsigned long int));
-typedef unsigned long int (*ddsi_div_func) __GMP_PROTO ((mpz_ptr, mpz_ptr, mpz_srcptr, unsigned long int));
-typedef void (*ddss_div_func) __GMP_PROTO ((mpz_ptr, mpz_ptr, mpz_srcptr, mpz_srcptr));
-typedef void (*ds_func) __GMP_PROTO ((mpz_ptr, mpz_srcptr));
+typedef void (*dss_func) (mpz_ptr, mpz_srcptr, mpz_srcptr);
+typedef void (*dsi_func) (mpz_ptr, mpz_srcptr, unsigned long int);
+typedef unsigned long int (*dsi_div_func) (mpz_ptr, mpz_srcptr, unsigned long int);
+typedef unsigned long int (*ddsi_div_func) (mpz_ptr, mpz_ptr, mpz_srcptr, unsigned long int);
+typedef void (*ddss_div_func) (mpz_ptr, mpz_ptr, mpz_srcptr, mpz_srcptr);
+typedef void (*ds_func) (mpz_ptr, mpz_srcptr);
 
 
 void
@@ -67,125 +69,140 @@ mpz_xinvert (mpz_ptr r, mpz_srcptr a, mpz_srcptr b)
     mpz_set_ui (r, 0);
 }
 
-dss_func dss_funcs[] =
-{
-  mpz_add, mpz_sub, mpz_mul,
-  mpz_cdiv_q, mpz_cdiv_r, mpz_fdiv_q, mpz_fdiv_r, mpz_tdiv_q, mpz_tdiv_r,
-  mpz_xinvert,
-  mpz_gcd, mpz_lcm, mpz_and, mpz_ior, mpz_xor
-};
-char *dss_func_names[] =
-{
-  "mpz_add", "mpz_sub", "mpz_mul",
-  "mpz_cdiv_q", "mpz_cdiv_r", "mpz_fdiv_q", "mpz_fdiv_r", "mpz_tdiv_q", "mpz_tdiv_r",
-  "mpz_xinvert",
-  "mpz_gcd", "mpz_lcm", "mpz_and", "mpz_ior", "mpz_xor"
-};
-char dss_func_division[] = {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0};
+struct {
+  dss_func fptr;
+  const char *fname;
+  int isdivision;
+  int isslow;
+} dss[] =
+  { { mpz_add,     "mpz_add",	  0, 0 },
+    { mpz_sub,     "mpz_sub",	  0, 0 },
+    { mpz_mul,     "mpz_mul",	  0, 0 },
+    { mpz_cdiv_q,  "mpz_cdiv_q",  1, 0 },
+    { mpz_cdiv_r,  "mpz_cdiv_r",  1, 0 },
+    { mpz_fdiv_q,  "mpz_fdiv_q",  1, 0 },
+    { mpz_fdiv_r,  "mpz_fdiv_r",  1, 0 },
+    { mpz_tdiv_q,  "mpz_tdiv_q",  1, 0 },
+    { mpz_tdiv_r,  "mpz_tdiv_r",  1, 0 },
+    { mpz_mod,     "mpz_mod",	  1, 0 },
+    { mpz_xinvert, "mpz_xinvert", 1, 1 },
+    { mpz_gcd,     "mpz_gcd",	  0, 1 },
+    { mpz_lcm,     "mpz_lcm",	  0, 1 },
+    { mpz_and,     "mpz_and",	  0, 0 },
+    { mpz_ior,     "mpz_ior",	  0, 0 },
+    { mpz_xor,     "mpz_xor",     0, 0 }
+  };
 
-dsi_func dsi_funcs[] =
+
+struct {
+  dsi_func fptr;
+  const char *fname;
+  int mod;
+} dsi[] =
 {
   /* Don't change order here without changing the code in main(). */
-  mpz_add_ui, mpz_mul_ui, mpz_sub_ui,
-  mpz_fdiv_q_2exp, mpz_fdiv_r_2exp,
-  mpz_cdiv_q_2exp, mpz_cdiv_r_2exp,
-  mpz_tdiv_q_2exp, mpz_tdiv_r_2exp,
-  mpz_mul_2exp,
-  mpz_pow_ui
-};
-char *dsi_func_names[] =
-{
-  "mpz_add_ui", "mpz_mul_ui", "mpz_sub_ui",
-  "mpz_fdiv_q_2exp", "mpz_fdiv_r_2exp",
-  "mpz_cdiv_q_2exp", "mpz_cdiv_r_2exp",
-  "mpz_tdiv_q_2exp", "mpz_tdiv_r_2exp",
-  "mpz_mul_2exp",
-  "mpz_pow_ui"
+  { mpz_add_ui,         "mpz_add_ui",	     0 },
+  { mpz_mul_ui,		"mpz_mul_ui",	     0 },
+  { mpz_sub_ui,		"mpz_sub_ui",	     0 },
+  { mpz_fdiv_q_2exp,    "mpz_fdiv_q_2exp",   0x1000 },
+  { mpz_fdiv_r_2exp,    "mpz_fdiv_r_2exp",   0x1000 },
+  { mpz_cdiv_q_2exp,    "mpz_cdiv_q_2exp",   0x1000 },
+  { mpz_cdiv_r_2exp,    "mpz_cdiv_r_2exp",   0x1000 },
+  { mpz_tdiv_q_2exp,    "mpz_tdiv_q_2exp",   0x1000 },
+  { mpz_tdiv_r_2exp,    "mpz_tdiv_r_2exp",   0x1000 },
+  { mpz_mul_2exp,	"mpz_mul_2exp",      0x100 },
+  { mpz_pow_ui,		"mpz_pow_ui",        0x10 }
 };
 
-dsi_div_func dsi_div_funcs[] =
+struct {
+  dsi_div_func fptr;
+  const char *fname;
+} dsi_div[] =
 {
-  mpz_cdiv_q_ui, mpz_cdiv_r_ui,
-  mpz_fdiv_q_ui, mpz_fdiv_r_ui,
-  mpz_tdiv_q_ui, mpz_tdiv_r_ui
-};
-char *dsi_div_func_names[] =
-{
-  "mpz_cdiv_q_ui", "mpz_cdiv_r_ui",
-  "mpz_fdiv_q_ui", "mpz_fdiv_r_ui",
-  "mpz_tdiv_q_ui", "mpz_tdiv_r_ui"
+  { mpz_cdiv_q_ui,       "mpz_cdiv_q_ui" },
+  { mpz_cdiv_r_ui,       "mpz_cdiv_r_ui" },
+  { mpz_fdiv_q_ui,       "mpz_fdiv_q_ui" },
+  { mpz_fdiv_r_ui,       "mpz_fdiv_r_ui" },
+  { mpz_tdiv_q_ui,       "mpz_tdiv_q_ui" },
+  { mpz_tdiv_r_ui,       "mpz_tdiv_r_ui" }
 };
 
-ddsi_div_func ddsi_div_funcs[] =
+struct {
+  ddsi_div_func fptr;
+  const char *fname;
+  int isslow;
+} ddsi_div[] =
 {
-  mpz_cdiv_qr_ui,
-  mpz_fdiv_qr_ui,
-  mpz_tdiv_qr_ui
-};
-char *ddsi_div_func_names[] =
-{
-  "mpz_cdiv_qr_ui",
-  "mpz_fdiv_qr_ui",
-  "mpz_tdiv_qr_ui"
-};
-
-ddss_div_func ddss_div_funcs[] =
-{
-  mpz_cdiv_qr,
-  mpz_fdiv_qr,
-  mpz_tdiv_qr
-};
-char *ddss_div_func_names[] =
-{
-  "mpz_cdiv_qr",
-  "mpz_fdiv_qr",
-  "mpz_tdiv_qr"
-};
-
-ds_func ds_funcs[] =
-{
-  mpz_abs, mpz_com, mpz_neg, mpz_sqrt
-};
-char *ds_func_names[] =
-{
-  "mpz_abs", "mpz_com", "mpz_neg", "mpz_sqrt"
+  { mpz_cdiv_qr_ui,     "mpz_cdiv_qr_ui",    0 },
+  { mpz_fdiv_qr_ui,     "mpz_fdiv_qr_ui",    0 },
+  { mpz_tdiv_qr_ui,     "mpz_tdiv_qr_ui",    0 },
 };
 
 
-/* Really use `defined (__STDC__)' here; we want it to be true for Sun C */
-#if defined (__STDC__) || defined (__cplusplus)
-#define FAIL(class,indx,op1,op2,op3) \
+struct {
+  ddss_div_func fptr;
+  const char *fname;
+  int isslow;
+} ddss_div[] =
+{
+  { mpz_cdiv_qr,  "mpz_cdiv_qr",    0 },
+  { mpz_fdiv_qr,  "mpz_fdiv_qr",    0 },
+  { mpz_tdiv_qr,  "mpz_tdiv_qr",    0 },
+};
+
+struct {
+  ds_func fptr;
+  const char *fname;
+  int nonneg;
+} ds[] =
+{
+  { mpz_abs,    "mpz_abs",    0 },
+  { mpz_com,    "mpz_com",    0 },
+  { mpz_neg,    "mpz_neg",    0 },
+  { mpz_sqrt,   "mpz_sqrt",   1 },
+};
+
+#define FAIL(class,indx,op1,op2,op3)					\
   do {									\
-  class##_funcs[indx] = 0;						\
-  dump (class##_func_names[indx], op1, op2, op3);			\
-  failures++;								\
+    dump (class[indx].fname, op1, op2, op3);				\
+    exit (1);								\
   } while (0)
-#define FAIL2(fname,op1,op2,op3) \
+
+#define FAIL2(fname,op1,op2,op3)					\
   do {									\
-  dump (#fname, op1, op2, op3);						\
-  failures++;								\
+    dump (#fname, op1, op2, op3);					\
+    exit (1);								\
   } while (0)
-#else
-#define FAIL(class,indx,op1,op2,op3) \
-  do {									\
-  class/**/_funcs[indx] = 0;						\
-  dump (class/**/_func_names[indx], op1, op2, op3);			\
-  failures++;								\
-  } while (0)
-#define FAIL2(fname,op1,op2,op3) \
-  do {									\
-  dump ("fname", op1, op2, op3);					\
-  failures++;								\
-  } while (0)
-#endif
 
 
+#define INVOKE_RRS(desc,r1,r2,i1)					\
+  do {									\
+    if (pass & 1) _mpz_realloc (r1, ABSIZ(r1));				\
+    if (pass & 2) _mpz_realloc (r2, ABSIZ(r2));				\
+    (desc).fptr (r1, r2, i1);						\
+  } while (0)
+#define INVOKE_RS(desc,r1,i1)						\
+  do {									\
+    if (pass & 1) _mpz_realloc (r1, ABSIZ(r1));				\
+    (desc).fptr (r1, i1);						\
+  } while (0)
+#define INVOKE_RRSS(desc,r1,r2,i1,i2)					\
+  do {									\
+    if (pass & 1) _mpz_realloc (r1, ABSIZ(r1));				\
+    if (pass & 2) _mpz_realloc (r2, ABSIZ(r2));				\
+    (desc).fptr (r1, r2, i1, i2);					\
+  } while (0)
+#define INVOKE_RSS(desc,r1,i1,i2)					\
+  do {									\
+    if (pass & 1) _mpz_realloc (r1, ABSIZ(r1));				\
+    (desc).fptr (r1, i1, i2);						\
+  } while (0)
 
 int
 main (int argc, char **argv)
 {
   int i;
-  int pass, reps = 100;
+  int pass, reps = 400;
   mpz_t in1, in2, in3;
   unsigned long int in2i;
   mp_size_t size;
@@ -193,7 +210,6 @@ main (int argc, char **argv)
   mpz_t ref1, ref2, ref3;
   mpz_t t;
   unsigned long int r1, r2;
-  long failures = 0;
   gmp_randstate_ptr rands;
   mpz_t bs;
   unsigned long bsi, size_range;
@@ -218,106 +234,126 @@ main (int argc, char **argv)
 
   for (pass = 1; pass <= reps; pass++)
     {
+      if (isatty (fileno (stdout)))
+	{
+	  printf ("\r%d/%d passes", pass, reps);
+	  fflush (stdout);
+	}
+
       mpz_urandomb (bs, rands, 32);
-      size_range = mpz_get_ui (bs) % 17 + 2;
+      size_range = mpz_get_ui (bs) % 21 + 2;
 
-      mpz_urandomb (bs, rands, size_range);
-      size = mpz_get_ui (bs);
-      mpz_rrandomb (in1, rands, size);
+      if ((pass & 1) == 0)
+	{
+	  /* Make all input operands have quite different sizes */
+	  mpz_urandomb (bs, rands, 32);
+	  size = mpz_get_ui (bs) % size_range;
+	  mpz_rrandomb (in1, rands, size);
 
-      mpz_urandomb (bs, rands, size_range);
-      size = mpz_get_ui (bs);
-      mpz_rrandomb (in2, rands, size);
+	  mpz_urandomb (bs, rands, 32);
+	  size = mpz_get_ui (bs) % size_range;
+	  mpz_rrandomb (in2, rands, size);
 
-      mpz_urandomb (bs, rands, size_range);
-      size = mpz_get_ui (bs);
-      mpz_rrandomb (in3, rands, size);
+	  mpz_urandomb (bs, rands, 32);
+	  size = mpz_get_ui (bs) % size_range;
+	  mpz_rrandomb (in3, rands, size);
+	}
+      else
+	{
+	  /* Make all input operands have about the same size */
+	  mpz_urandomb (bs, rands, size_range);
+	  size = mpz_get_ui (bs);
+	  mpz_rrandomb (in1, rands, size);
+
+	  mpz_urandomb (bs, rands, size_range);
+	  size = mpz_get_ui (bs);
+	  mpz_rrandomb (in2, rands, size);
+
+	  mpz_urandomb (bs, rands, size_range);
+	  size = mpz_get_ui (bs);
+	  mpz_rrandomb (in3, rands, size);
+	}
 
       mpz_urandomb (bs, rands, 3);
       bsi = mpz_get_ui (bs);
       if ((bsi & 1) != 0)
 	mpz_neg (in1, in1);
-      if ((bsi & 1) != 0)
+      if ((bsi & 2) != 0)
 	mpz_neg (in2, in2);
-      if ((bsi & 1) != 0)
+      if ((bsi & 4) != 0)
 	mpz_neg (in3, in3);
 
-      for (i = 0; i < sizeof (dss_funcs) / sizeof (dss_func); i++)
+      for (i = 0; i < numberof (dss); i++)
 	{
-	  if (dss_funcs[i] == 0)
+	  if (dss[i].isdivision && mpz_sgn (in2) == 0)
 	    continue;
-	  if (dss_func_division[i] && mpz_sgn (in2) == 0)
+	  if (dss[i].isslow && size_range > 19)
 	    continue;
 
-	  (dss_funcs[i]) (ref1, in1, in2);
+	  (dss[i].fptr) (ref1, in1, in2);
 	  MPZ_CHECK_FORMAT (ref1);
 
 	  mpz_set (res1, in1);
-	  (dss_funcs[i]) (res1, res1, in2);
+	  INVOKE_RSS (dss[i], res1, res1, in2);
 	  MPZ_CHECK_FORMAT (res1);
 	  if (mpz_cmp (ref1, res1) != 0)
 	    FAIL (dss, i, in1, in2, NULL);
 
 	  mpz_set (res1, in2);
-	  (dss_funcs[i]) (res1, in1, res1);
+	  INVOKE_RSS (dss[i], res1, in1, res1);
 	  MPZ_CHECK_FORMAT (res1);
 	  if (mpz_cmp (ref1, res1) != 0)
 	    FAIL (dss, i, in1, in2, NULL);
 	}
 
-      for (i = 0; i < sizeof (ddss_div_funcs) / sizeof (ddss_div_func); i++)
+      for (i = 0; i < numberof (ddss_div); i++)
 	{
-	  if (ddss_div_funcs[i] == 0)
-	    continue;
 	  if (mpz_sgn (in2) == 0)
 	    continue;
 
-	  (ddss_div_funcs[i]) (ref1, ref2, in1, in2);
+	  (ddss_div[i].fptr) (ref1, ref2, in1, in2);
 	  MPZ_CHECK_FORMAT (ref1);
 	  MPZ_CHECK_FORMAT (ref2);
 
 	  mpz_set (res1, in1);
-	  (ddss_div_funcs[i]) (res1, res2, res1, in2);
+	  INVOKE_RRSS (ddss_div[i], res1, res2, res1, in2);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL (ddss_div, i, in1, in2, NULL);
 
 	  mpz_set (res2, in1);
-	  (ddss_div_funcs[i]) (res1, res2, res2, in2);
+	  INVOKE_RRSS (ddss_div[i], res1, res2, res2, in2);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL (ddss_div, i, in1, in2, NULL);
 
 	  mpz_set (res1, in2);
-	  (ddss_div_funcs[i]) (res1, res2, in1, res1);
+	  INVOKE_RRSS (ddss_div[i], res1, res2, in1, res1);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL (ddss_div, i, in1, in2, NULL);
 
 	  mpz_set (res2, in2);
-	  (ddss_div_funcs[i]) (res1, res2, in1, res2);
+	  INVOKE_RRSS (ddss_div[i], res1, res2, in1, res2);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL (ddss_div, i, in1, in2, NULL);
 	}
 
-      for (i = 0; i < sizeof (ds_funcs) / sizeof (ds_func); i++)
+      for (i = 0; i < numberof (ds); i++)
 	{
-	  if (ds_funcs[i] == 0)
-	    continue;
-	  if (strcmp (ds_func_names[i], "mpz_sqrt") == 0
-	      && mpz_sgn (in1) < 0)
+	  if (ds[i].nonneg && mpz_sgn (in1) < 0)
 	    continue;
 
-	  (ds_funcs[i]) (ref1, in1);
+	  (ds[i].fptr) (ref1, in1);
 	  MPZ_CHECK_FORMAT (ref1);
 
 	  mpz_set (res1, in1);
-	  (ds_funcs[i]) (res1, res1);
+	  INVOKE_RS (ds[i], res1, res1);
 	  MPZ_CHECK_FORMAT (res1);
 	  if (mpz_cmp (ref1, res1) != 0)
 	    FAIL (ds, i, in1, in2, NULL);
@@ -325,27 +361,16 @@ main (int argc, char **argv)
 
       in2i = mpz_get_ui (in2);
 
-      for (i = 0; i < sizeof (dsi_funcs) / sizeof (dsi_func); i++)
+      for (i = 0; i < numberof (dsi); i++)
 	{
-	  if (dsi_funcs[i] == 0)
-	    continue;
-	  if (strcmp (dsi_func_names[i], "mpz_fdiv_q_2exp") == 0)
-	    /* Limit exponent to something reasonable for the division
-	       functions.  Without this, we'd  normally shift things off
-	       the end and just generate the trivial values 1, 0, -1.  */
-	    in2i %= 0x1000;
-	  if (strcmp (dsi_func_names[i], "mpz_mul_2exp") == 0)
-	    /* Limit exponent more for mpz_mul_2exp to save time.  */
-	    in2i %= 0x100;
-	  if (strcmp (dsi_func_names[i], "mpz_pow_ui") == 0)
-	    /* Limit exponent yet more for mpz_pow_ui to save time.  */
-	    in2i %= 0x10;
+	  if (dsi[i].mod != 0)
+	    in2i = mpz_get_ui (in2) % dsi[i].mod;
 
-	  (dsi_funcs[i]) (ref1, in1, in2i);
+	  (dsi[i].fptr) (ref1, in1, in2i);
 	  MPZ_CHECK_FORMAT (ref1);
 
 	  mpz_set (res1, in1);
-	  (dsi_funcs[i]) (res1, res1, in2i);
+	  INVOKE_RRS (dsi[i], res1, res1, in2i);
 	  MPZ_CHECK_FORMAT (res1);
 	  if (mpz_cmp (ref1, res1) != 0)
 	    FAIL (dsi, i, in1, in2, NULL);
@@ -353,31 +378,31 @@ main (int argc, char **argv)
 
       if (in2i != 0)	  /* Don't divide by 0.  */
 	{
-	  for (i = 0; i < sizeof (dsi_div_funcs) / sizeof (dsi_div_funcs); i++)
+	  for (i = 0; i < numberof (dsi_div); i++)
 	    {
-	      r1 = (dsi_div_funcs[i]) (ref1, in1, in2i);
+	      r1 = (dsi_div[i].fptr) (ref1, in1, in2i);
 	      MPZ_CHECK_FORMAT (ref1);
 
 	      mpz_set (res1, in1);
-	      r2 = (dsi_div_funcs[i]) (res1, res1, in2i);
+	      r2 = (dsi_div[i].fptr) (res1, res1, in2i);
 	      MPZ_CHECK_FORMAT (res1);
 	      if (mpz_cmp (ref1, res1) != 0 || r1 != r2)
 		FAIL (dsi_div, i, in1, in2, NULL);
 	    }
 
-	  for (i = 0; i < sizeof (ddsi_div_funcs) / sizeof (ddsi_div_funcs); i++)
+	  for (i = 0; i < numberof (ddsi_div); i++)
 	    {
-	      r1 = (ddsi_div_funcs[i]) (ref1, ref2, in1, in2i);
+	      r1 = (ddsi_div[i].fptr) (ref1, ref2, in1, in2i);
 	      MPZ_CHECK_FORMAT (ref1);
 
 	      mpz_set (res1, in1);
-	      r2 = (ddsi_div_funcs[i]) (res1, res2, res1, in2i);
+	      r2 = (ddsi_div[i].fptr) (res1, res2, res1, in2i);
 	      MPZ_CHECK_FORMAT (res1);
 	      if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0 || r1 != r2)
 		FAIL (ddsi_div, i, in1, in2, NULL);
 
 	      mpz_set (res2, in1);
-	      (ddsi_div_funcs[i]) (res1, res2, res2, in2i);
+	      (ddsi_div[i].fptr) (res1, res2, res2, in2i);
 	      MPZ_CHECK_FORMAT (res1);
 	      if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0 || r1 != r2)
 		FAIL (ddsi_div, i, in1, in2, NULL);
@@ -403,15 +428,21 @@ main (int argc, char **argv)
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL2 (mpz_sqrtrem, in1, NULL, NULL);
+
+	  mpz_set (res1, in1);
+	  mpz_sqrtrem (res1, res1, res1);
+	  MPZ_CHECK_FORMAT (res1);
+	  if (mpz_cmp (ref2, res1) != 0)
+	    FAIL2 (mpz_sqrtrem, in1, NULL, NULL);
 	}
 
       if (mpz_sgn (in1) >= 0)
 	{
-	  mpz_root (ref1, in1, in2i % 0x1000 + 1);
+	  mpz_root (ref1, in1, in2i % 0x100 + 1);
 	  MPZ_CHECK_FORMAT (ref1);
 
 	  mpz_set (res1, in1);
-	  mpz_root (res1, res1, in2i % 0x1000 + 1);
+	  mpz_root (res1, res1, in2i % 0x100 + 1);
 	  MPZ_CHECK_FORMAT (res1);
 	  if (mpz_cmp (ref1, res1) != 0)
 	    FAIL2 (mpz_root, in1, in2, NULL);
@@ -419,26 +450,26 @@ main (int argc, char **argv)
 
       if (mpz_sgn (in1) >= 0)
 	{
-	  mpz_rootrem (ref1, ref2, in1, in2i % 0x1000 + 1);
+	  mpz_rootrem (ref1, ref2, in1, in2i % 0x100 + 1);
 	  MPZ_CHECK_FORMAT (ref1);
 	  MPZ_CHECK_FORMAT (ref2);
 
 	  mpz_set (res1, in1);
-	  mpz_rootrem (res1, res2, res1, in2i % 0x1000 + 1);
+	  mpz_rootrem (res1, res2, res1, in2i % 0x100 + 1);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL2 (mpz_rootrem, in1, in2, NULL);
 
 	  mpz_set (res2, in1);
-	  mpz_rootrem (res1, res2, res2, in2i % 0x1000 + 1);
+	  mpz_rootrem (res1, res2, res2, in2i % 0x100 + 1);
 	  MPZ_CHECK_FORMAT (res1);
 	  MPZ_CHECK_FORMAT (res2);
 	  if (mpz_cmp (ref1, res1) != 0 || mpz_cmp (ref2, res2) != 0)
 	    FAIL2 (mpz_rootrem, in1, in2, NULL);
 	}
 
-      if (pass < reps / 2)	/* run fewer tests since gcdext lots of time */
+      if (size_range < 18)	/* run fewer tests since gcdext lots of time */
 	{
 	  mpz_gcdext (ref1, ref2, ref3, in1, in2);
 	  MPZ_CHECK_FORMAT (ref1);
@@ -533,7 +564,7 @@ main (int argc, char **argv)
 	}
 
       /* Don't run mpz_powm for huge exponents or when undefined.  */
-      if (mpz_sizeinbase (in2, 2) < 250 && mpz_sgn (in3) != 0
+      if (size_range < 17 && mpz_sizeinbase (in2, 2) < 250 && mpz_sgn (in3) != 0
 	  && (mpz_sgn (in2) >= 0 || mpz_invert (t, in1, in3)))
 	{
 	  mpz_powm (ref1, in1, in2, in3);
@@ -559,7 +590,7 @@ main (int argc, char **argv)
 	}
 
       /* Don't run mpz_powm_ui when undefined.  */
-      if (mpz_sgn (in3) != 0)
+      if (size_range < 17 && mpz_sgn (in3) != 0)
 	{
 	  mpz_powm_ui (ref1, in1, in2i, in3);
 	  MPZ_CHECK_FORMAT (ref1);
@@ -588,22 +619,23 @@ main (int argc, char **argv)
 	  FAIL2 (mpz_gcd_ui, in1, in2, NULL);
       }
 
-      if (mpz_cmp_ui (in2, 1L) > 0 && mpz_sgn (in1) != 0)
+      if (mpz_sgn (in2) != 0)
 	{
 	  /* Test mpz_remove */
-	  mpz_remove (ref1, in1, in2);
+	  mp_bitcnt_t refretval, retval;
+	  refretval = mpz_remove (ref1, in1, in2);
 	  MPZ_CHECK_FORMAT (ref1);
 
 	  mpz_set (res1, in1);
-	  mpz_remove (res1, res1, in2);
+	  retval = mpz_remove (res1, res1, in2);
 	  MPZ_CHECK_FORMAT (res1);
-	  if (mpz_cmp (ref1, res1) != 0)
+	  if (mpz_cmp (ref1, res1) != 0 || refretval != retval)
 	    FAIL2 (mpz_remove, in1, in2, NULL);
 
 	  mpz_set (res1, in2);
-	  mpz_remove (res1, in1, res1);
+	  retval = mpz_remove (res1, in1, res1);
 	  MPZ_CHECK_FORMAT (res1);
-	  if (mpz_cmp (ref1, res1) != 0)
+	  if (mpz_cmp (ref1, res1) != 0 || refretval != retval)
 	    FAIL2 (mpz_remove, in1, in2, NULL);
 	}
 
@@ -648,11 +680,8 @@ main (int argc, char **argv)
 	}
     }
 
-  if (failures != 0)
-    {
-      fprintf (stderr, "mpz/reuse: %ld error%s\n", failures, "s" + (failures == 1));
-      exit (1);
-    }
+  if (isatty (fileno (stdout)))
+    printf ("\r%20s", "");
 
   mpz_clear (bs);
   mpz_clear (in1);
@@ -666,24 +695,27 @@ main (int argc, char **argv)
   mpz_clear (res3);
   mpz_clear (t);
 
+  if (isatty (fileno (stdout)))
+    printf ("\r");
+
   tests_end ();
   exit (0);
 }
 
 void
-dump (char *name, mpz_t in1, mpz_t in2, mpz_t in3)
+dump (const char *name, mpz_t in1, mpz_t in2, mpz_t in3)
 {
   printf ("failure in %s (", name);
-  mpz_out_str (stdout, -16, in1);
+  0 && mpz_out_str (stdout, -16, in1);
   if (in2 != NULL)
     {
       printf (" ");
-      mpz_out_str (stdout, -16, in2);
+      0 && mpz_out_str (stdout, -16, in2);
     }
   if (in3 != NULL)
     {
       printf (" ");
-      mpz_out_str (stdout, -16, in3);
+      0 && mpz_out_str (stdout, -16, in3);
     }
   printf (")\n");
 }
