@@ -2,25 +2,22 @@
    number sequences. */
 
 /*
-Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+Copyright 1999, 2000 Free Software Foundation, Inc.
 
-This file is part of the GNU MP Library.
+This file is part of the GNU MP Library test suite.
 
-The GNU MP Library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
-option) any later version.
+The GNU MP Library test suite is free software; you can redistribute it
+and/or modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 3 of the License,
+or (at your option) any later version.
 
-The GNU MP Library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-License for more details.
+The GNU MP Library test suite is distributed in the hope that it will be
+useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
+Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA.
-*/
+You should have received a copy of the GNU General Public License along with
+the GNU MP Library test suite.  If not, see http://www.gnu.org/licenses/.  */
 
 /* The theories for these functions are taken from D. Knuth's "The Art
 of Computer Programming: Volume 2, Seminumerical Algorithms", Third
@@ -63,9 +60,9 @@ for (each observation Xj)
 			rm = max (rm, a[k] - j/n)
 			j += c[k]
 			rp = max (rp, j/n - b[k])
-			
+
 Kp = sqr (n) * rp
-Km = sqr (n) * rm 
+Km = sqr (n) * rm
 
 */
 
@@ -97,15 +94,15 @@ ks (mpf_t Kp,
   mpf_t f_jnq;			/* j/n or (j-1)/n */
   unsigned long int j;
 
-  /* Sort the vector in ascending order. */  
+  /* Sort the vector in ascending order. */
   qsort (X, n, sizeof (__mpf_struct), mpf_cmp);
 
   /* K-S test. */
-  /* 	Kp = sqr(n) * max(j/n - F(Xj))		for all 1<=j<=n
+  /*	Kp = sqr(n) * max(j/n - F(Xj))		for all 1<=j<=n
 	Km = sqr(n) * max(F(Xj) - (j-1)/n))	for all 1<=j<=n
   */
 
-  mpf_init (Kt); mpf_init (f_x); mpf_init (f_j); mpf_init (f_jnq); 
+  mpf_init (Kt); mpf_init (f_x); mpf_init (f_j); mpf_init (f_jnq);
   mpf_set_ui (Kp, 0);  mpf_set_ui (Km, 0);
   for (j = 1; j <= n; j++)
     {
@@ -143,7 +140,7 @@ ks (mpf_t Kp,
   mpf_mul (Kp, Kp, Kt);
   mpf_mul (Km, Km, Kt);
 
-  mpf_clear (Kt); mpf_clear (f_x); mpf_clear (f_j); mpf_clear (f_jnq); 
+  mpf_clear (Kt); mpf_clear (f_x); mpf_clear (f_j); mpf_clear (f_jnq);
 }
 
 /* ks_table(val, n) -- calculate probability for Kp/Km less than or
@@ -178,7 +175,7 @@ ks_table (mpf_t p, mpf_t val, const unsigned int n)
 #else
   /* hmmm, gmp doesn't have pow() for floats.  use doubles. */
   mpf_set_d (t2, pow (M_E, -(2 * pow (mpf_get_d (val), 2))));
-#endif  
+#endif
 
   /* p = 1 - t1 * t2 */
   mpf_mul (t1, t1, t2);
@@ -207,7 +204,7 @@ x2_table (double t[],
 
   /* value = v + sqrt(2*v) * X[p] + (2/3) * X[p]^2 - 2/3 + O(1/sqrt(t) */
   /* NOTE: The O() term is ignored for simplicity. */
-  
+
   for (f = 0; f < 7; f++)
       t[f] =
 	v +
@@ -221,7 +218,7 @@ being greater than or equal to any number in the sequence.  For a
 random real number between zero and one given by a uniformly
 distributed random number generator, this is simply equal to X. */
 
-static void 
+static void
 P (mpf_t p, mpf_t x)
 {
   mpf_set (p, x);
@@ -282,7 +279,7 @@ x2 (mpf_t V,			/* result */
     fprintf (stderr, "\n");
   mpf_div_ui (V, V, n);
   mpf_sub_ui (V, V, n);
-  
+
   mpf_clear (f_t); mpf_clear (f_t2);
 }
 
@@ -301,7 +298,7 @@ Pzf (mpf_t p, unsigned long int s, void *x)
 
    X[] must not contain numbers outside the range 0 <= X <= IMAX.
 
-   Return value is number of observations actally used, after
+   Return value is number of observations actually used, after
    discarding entries out of range.
 
    Since X[] contains integers between zero and IMAX, inclusive, we
@@ -364,14 +361,14 @@ mpz_freqt (mpf_t V,
 
 /* debug dummy to drag in dump funcs */
 void
-foo_debug () 
+foo_debug ()
 {
   if (0)
     {
-      mpf_dump (0); 
+      mpf_dump (0);
 #ifndef OLDGMP
       mpz_dump (0);
-#endif      
+#endif
     }
 }
 
@@ -388,7 +385,7 @@ merit (mpf_t rop, unsigned int t, mpf_t v, mpz_t m)
   mpf_set_z (f_m, m);
   mpf_init_set_d (f_const, M_PI);
   mpf_init_set_d (f_pi, M_PI);
-  
+
   switch (t)
     {
     case 2:			/* PI */
@@ -435,7 +432,7 @@ merit_u (unsigned int t, mpf_t v, mpz_t m)
 {
   mpf_t rop;
   double res;
-  
+
   mpf_init (rop);
   merit (rop, t, v, m);
   res = mpf_get_d (rop);
@@ -533,13 +530,13 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 
   /* S1 [Initialize.] */
   ui_t = 2 - 1;			/* NOTE: `t' in description == ui_t + 1
-                                   for easy indexing */
+				   for easy indexing */
   mpz_set (h, a);
   mpz_set (hp, m);
   mpz_set_ui (p, 1);
   mpz_set_ui (pp, 0);
   mpz_set (r, a);
-  mpz_pow_ui (s, a, 2);		
+  mpz_pow_ui (s, a, 2);
   mpz_add_ui (s, s, 1);		/* s = 1 + a^2 */
 
   /* S2 [Euclidean step.] */
@@ -585,7 +582,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 
       mpz_mul (tmp1, q, p);
       mpz_sub (v, pp, tmp1);	/* v = pp - q*p */
-  
+
       mpz_pow_ui (tmp1, u, 2);
       mpz_pow_ui (tmp2, v, 2);
       mpz_add (tmp1, tmp1, tmp2);
@@ -616,13 +613,13 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
     }
   mpf_set_z (f_tmp1, s);
   mpf_sqrt (rop[ui_t - 1], f_tmp1);
-      
+
   /* S4 [Advance t.] */
   mpz_neg (U[0][0], h);
   mpz_set (U[0][1], p);
   mpz_neg (U[1][0], hp);
   mpz_set (U[1][1], pp);
-  
+
   mpz_set (V[0][0], pp);
   mpz_set (V[0][1], hp);
   mpz_neg (V[1][0], p);
@@ -648,7 +645,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
       mpz_set_ui (U[ui_t][ui_t], 1); /* U: Last col in new row. */
 
       mpz_set (V[ui_t][ui_t], m); /* V: Last col in new row. */
-      
+
       /* "Finally, for 1 <= i < t,
 	   set q = round (vi1 * r / m),
 	   vit = vi1*r - q*m,
@@ -679,7 +676,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
       /* S5 [Transform.] */
       if (g_debug > DEBUG_2)
 	printf ("(t, k, j, q1, q2, ...)\n");
-      do 
+      do
 	{
 	  if (g_debug > DEBUG_2)
 	    printf ("(%u, %u, %u", ui_t + 1, ui_k + 1, ui_j + 1);
@@ -709,7 +706,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 			  mpz_mul (tmp1, q, U[ui_i][ui_l]);
 			  mpz_add (U[ui_j][ui_l], U[ui_j][ui_l], tmp1); /* Uj=Uj+q*Ui */
 			}
-		      
+
 		      vz_dot (tmp1, U[ui_j], U[ui_j], ui_t + 1); /* tmp1=dot(Uj,Uj) */
 		      if (mpz_cmp (tmp1, s) < 0) /* s = min(s,dot(Uj,Uj)) */
 			mpz_set (s, tmp1);
@@ -735,7 +732,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 
       /* From Knuth p. 104: "The exhaustive search in steps S8-S10
 	 reduces the value of s only rarely." */
-#ifdef DO_SEARCH	
+#ifdef DO_SEARCH
       /* S7 [Prepare for search.] */
       /* Find minimum in (x[1], ..., x[t]) satisfying condition
 	 x[k]^2 <= f(y[1], ...,y[t]) * dot(V[k],V[k]) */
@@ -764,7 +761,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 	}
 
       /* S8 [Advance X[k].] */
-      do 
+      do
 	{
 	  if (g_debug > DEBUG_2)
 	    {
@@ -775,7 +772,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 	      printf ("\n");
 	      fflush (stdout);
 	    }
-	      
+
 	  if (mpz_cmp (X[ui_k], Z[ui_k]))
 	    {
 	      mpz_add_ui (X[ui_k], X[ui_k], 1);
@@ -802,7 +799,7 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 #endif /* DO_SEARCH */
       mpf_set_z (f_tmp1, s);
       mpf_sqrt (rop[ui_t - 1], f_tmp1);
-#ifdef DO_SEARCH	
+#ifdef DO_SEARCH
       if (g_debug > DEBUG_2)
 	printf ("done.\n");
 #endif /* DO_SEARCH */
@@ -838,4 +835,3 @@ spectral_test (mpf_t rop[], unsigned int T, mpz_t a, mpz_t m)
 
   return;
 }
-
